
/*
#include <Nexgen_Rover_v4.h>
If I include the above, sensors read 0 all the time
*/
#include <QTRSensors.h>

#define DEFAULT_SPEED_LEFT 160
#define DEFAULT_SPEED_RIGHT 160
#define OPTIMUM_L_MOTOR_LEFT_TURN_SPEED 110
#define OPTIMUM_R_MOTOR_LEFT_TURN_SPEED 255
#define OPTIMUM_L_MOTOR_RIGHT_TURN_SPEED 255
#define OPTIMUM_R_MOTOR_RIGHT_TURN_SPEED 110

QTRSensors qtr;

int motorLPin = 3;                   // PWM output pin connected to the motor
int dirLPin = 2;                     // direction pin connected to the motor
int motorRPin = 6;                   // PWM output pin connected to the motor
int dirRPin = 5;                     // direction pin connected to the motor
int speed = 0;                       // speed value between 0 and 255
int buzzerPin = 8;

void setup() {
    Serial.begin(9600);

    pinMode(buzzerPin, OUTPUT);
    pinMode(motorLPin, OUTPUT);
    pinMode(dirLPin, OUTPUT);
    pinMode(motorRPin, OUTPUT);
    pinMode(dirRPin, OUTPUT);
    
    // Initialize the sensors & calibrate the line sensors
    initSensors();
    calibrate();
}

void loop() {
    unsigned int sensors[3];

    // Get calibrated sensor values and return position. 1000=CENTRE
    int position = qtr.readLineBlack(sensors);

    // Categorise the position
    boolean isOnLine = position > 600 && position < 1400;
    boolean isLeftOfLine = position >= 0 && position <= 600;
    boolean isRightOfLine = position >= 1400 && position <= 2000;

    log(sensors, isOnLine, isLeftOfLine, isRightOfLine, position, true);

    if (isOnLine) {
        forward(DEFAULT_SPEED_LEFT, DEFAULT_SPEED_RIGHT);
    }
    else if (isLeftOfLine) {
        turnRight();
    }
    else if (isRightOfLine) {
        turnLeft();
    }
    else {
        stop();
    }
}

void turnLeft() {
    setDirection(LOW, HIGH);

    // Left motor much slower than right
    setSpeed(OPTIMUM_L_MOTOR_LEFT_TURN_SPEED, OPTIMUM_R_MOTOR_LEFT_TURN_SPEED);
    stop();
}

void turnRight() {
    setDirection(HIGH, LOW);

    // Right motor much slower than left
    setSpeed(OPTIMUM_L_MOTOR_RIGHT_TURN_SPEED, OPTIMUM_R_MOTOR_RIGHT_TURN_SPEED);
    stop();
}

void forward(int speedL, int speedR) {
    setDirection(HIGH, HIGH);
    setSpeedLeft(speedL);
    setSpeedRight(speedR);
}

void forward(int speed) {
    setDirection(HIGH, HIGH);
    setSpeed(speed);
}

void setSpeed(int sp) {
    // set the motor speed to a value between 0 and 255
    analogWrite(motorLPin, sp);
    analogWrite(motorRPin, sp);
}

void setSpeed(int sp_left, int sp_right) {
    // set the motor speed to a value between 0 and 255
    analogWrite(motorLPin, sp_left);
    analogWrite(motorRPin, sp_right);
}

void setSpeedLeft(int sp_left) {
    // set the motor speed to a value between 0 and 255
    analogWrite(motorLPin, sp_left);
}

void setSpeedRight(int sp_right) {
    // set the motor speed to a value between 0 and 255
    analogWrite(motorRPin, sp_right);
}

void setDirection(int dLeft, int dRight) {
    digitalWrite(dirLPin, dLeft);
    digitalWrite(dirRPin, dRight);
}

void stop() {
    delay(5);
    setSpeed(0);
}

void playTone(int freq) {
    tone(buzzerPin, freq);
    delay(1000); // Wait for 1 second
    noTone(buzzerPin);
}

void initSensors() {
    qtr.setTypeRC(); // or setTypeAnalog()
    qtr.setSensorPins((const uint8_t[]) { A1, A2, A3 }, 3);
}

void calibrate() {

    playTone(2000);
    Serial.println("###");
    Serial.println("### STARTED CALIBRATION ###");
    Serial.println("### CALIBRATION IS NECESSARY BEFORE USING readLineBlack/White OR IT WILL NOT RETURN CORRECT VALUES ###");
    Serial.println("###");

    for (int i = 0; i < 250; i++) {
        qtr.calibrate();
        delay(20);
    }

    Serial.println("###");
    Serial.println("### FINISHED CALIBRATION ###");
    Serial.println("###");

    // Play a tone at 1000 Hz (1 kHz) for 1000 milliseconds (1 second)
    playTone(1000);
}

void log(unsigned int sensors[], boolean isOnLine, boolean isLeftOfLine, boolean isRightOfLine, int position, boolean doLog) {
    if (doLog) {
        Serial.print(sensors[0]);Serial.print(", ");Serial.print(sensors[1]);Serial.print(", ");Serial.println(sensors[2]);
        Serial.print(isOnLine);Serial.print(", ");Serial.print(isLeftOfLine);Serial.print(", ");Serial.println(isRightOfLine);
        Serial.print("Position: ");Serial.println(position);
    }
}



























